﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Merthsoft.CALCnet {
	/// <summary>
	/// A broadcast packet.
	/// </summary>
	public class BroadcastPacket : CALCnetPacket {
		protected static byte[] BroadcastID = new byte[] { 0, 0, 0, 0, 0 };
		protected static byte[] Header = new byte[] { 255, 137 };
		protected static byte Footer = 42;
		
		/// <summary>
		/// Gets the calculator ID
		/// </summary>
		public byte[] CalcID {
			get;
			protected set;
		}

		/// <summary>
		/// Gets the data associated with this packet. Modifying will not change the packet in any way.
		/// </summary>
		public byte[] Data {
			get;
			protected set;
		}

		/// <summary>
		/// Creates a new BroadcastPacket from the given calcID and data
		/// </summary>
		/// <param name="calcID">The calc id of the calc sending.</param>
		/// <param name="data">The data to be put in to the packet.</param>
		public BroadcastPacket(ICollection<byte> calcID, ICollection<byte> data) {
			if (calcID.Count != 5) {
				throw new ArgumentException("calcID must be 5 bytes long");
			}
			CalcID = calcID.ToArray();
			Data = data.ToArray();
			Type = PacketType.Broadcast;
			List<byte> payload = new List<byte>();
			payload.AddRange(Header);
			payload.AddRange(BroadcastID);
			payload.AddRange(calcID);
			payload.AddRange(((short)data.Count).GetBytes());
			payload.AddRange(data);
			payload.Add(Footer);

			Payload = payload;
		}
	}
}
