﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Merthsoft.CALCnet {
	/// <summary>
	/// Represents a CALCnetPacket
	/// </summary>
	public class CALCnetPacket {
		/// <summary>
		/// That types of packets one can send and/or receive
		/// </summary>
		public enum PacketType { Calculator = 'c', Broadcast = 'b', Directed = 'f', Join = 'j' }

		/// <summary>
		/// Gets or sets the size of the packet
		/// </summary>
		public int Size { get; set; }

		/// <summary>
		/// Gets or sets the type of the packet
		/// </summary>
		public PacketType Type { get; set; }

		protected List<byte> _payLoad;
		/// <summary>
		/// Gets or sets the payload of the packet, updating the size.
		/// </summary>
		public List<byte> Payload {
			get { return _payLoad; }
			set {
				_payLoad = value;
				Size = _payLoad.Count;
			}
		}

		/// <summary>
		/// Initializes and empty calcnetpacket
		/// </summary>
		public CALCnetPacket() {
			_payLoad = new List<byte>();
		}

		/// <summary>
		/// Get the raw byte representation of the packet
		/// </summary>
		/// <returns>A byte array that is the raw byte representation of the packet</returns>
		public byte[] ToFormedPacket() {
			List<byte> packet = new List<byte>();
			packet.Add((byte)(Size & 0xFF));
			packet.Add((byte)((Size & 0xFF00) >> 8));
			packet.Add((byte)(char)Type);
			packet.AddRange(_payLoad);

			return packet.ToArray();
		}

		/// <summary>
		/// Gets the raw-byte representation of the packet as a string.
		/// </summary>
		/// <returns>A string that is the raw-byte representation of the packet</returns>
		public override string ToString() {
			return ASCIIEncoding.ASCII.GetString(_payLoad.ToArray());
		}
	}
}
