﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


namespace Merthsoft.CALCnet {
	/// <summary>
	/// Creates packets
	/// </summary>
	public static class PacketFactory {
		/// <summary>
		/// Gets a specific packet from the generic packet passed in.
		/// This is used to transform generic packets that are received into easier to manipulate packets
		/// with a strong type.
		/// </summary>
		/// <param name="packet">The generic packet to transform</param>
		/// <returns>A new packet that is the specific type of the generic one passed in.</returns>
		public static CALCnetPacket GetSpecificPacket(CALCnetPacket packet) {
			CALCnetPacket retPacket = null;
			byte[] payload = packet.Payload.ToArray();
			switch (packet.Type) {
				case CALCnetPacket.PacketType.Broadcast:
					byte[] calcID = payload.SubArray(7, 5);
					short len = BitConverter.ToInt16(payload, 12);//payload[12];
					byte[] data = payload.SubArray(14, len);
					retPacket = new BroadcastPacket(calcID, data);
					break;
				case CALCnetPacket.PacketType.Calculator:
					calcID = new byte[5];
					for (int i = 0; i < payload.Length; i += 2) {
						calcID[i / 2] = byte.Parse(Encoding.ASCII.GetString(payload, i, 2));
					}
					break;
				case CALCnetPacket.PacketType.Directed:
					byte[] toID = payload.SubArray(2, 5);
					calcID = payload.SubArray(7, 5);
					len = BitConverter.ToInt16(payload, 12);//payload[12];
					data = payload.SubArray(14, len);
					retPacket = new DirectPacket(calcID, toID, data);
					break;
				case CALCnetPacket.PacketType.Join:
					byte hubLen = payload[0];
					string hubname = Encoding.ASCII.GetString(payload.SubArray(1, hubLen));
					byte localLen = payload[hubLen + 2];
					string localname = Encoding.ASCII.GetString(payload.SubArray(hubLen + 3, localLen));
					retPacket = new JoinPacket(hubname, localname);
					break;
				default:
					retPacket = null;
					break;
			}

			return retPacket;
		}
	}
}
