﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Merthsoft.CALCnet;


namespace Chat_ {
	public partial class Chat : Form {
		private class User {
			public string name;
			public DateTime lastCheckInTime;

			public User(string n, DateTime t) {
				name = n;
				lastCheckInTime = t;
			}
		}

		CALCnet c;
		List<byte> UserNameBytes;
		string UserName = "ShaunCom";
		Thread recvThread;
		Dictionary<string, User> users;
		BroadcastPacket broadcast;

		object userLock = new object();

		public Chat() {
			InitializeComponent();
			users = new Dictionary<string, User>();
		}

		private void Write(string text) {
			if (this.IsDisposed)
				return;
			SetTextCallback d = new SetTextCallback(SetText);
			this.Invoke(d, new object[] { text });
		}
		delegate void SetTextCallback(string text);
		private void SetText(string text) {
				this.chatBox.AppendText(text + Environment.NewLine);
		}

		private void AddName(string text) {
			if (this.nameBox.InvokeRequired) {
				// It's on a different thread, so use Invoke.
				AddNameCallBack d = new AddNameCallBack(SetName);
				this.Invoke
					(d, new object[] { text });
			} else {
				// It's on the same thread, no need for Invoke
				this.chatBox.AppendText(text + Environment.NewLine);
			}
		}
		delegate void AddNameCallBack(string text);
		// This method is passed in to the SetTextCallBack delegate
		// to set the Text property of textBox1.
		private void SetName(string text) {
			this.nameBox.Items.Add(text);
		}

		private void RemoveName(string text) {
			if (this.nameBox.InvokeRequired) {
				// It's on a different thread, so use Invoke.
				RemoveNameCallBack d = new RemoveNameCallBack(DelName);
				this.Invoke
					(d, new object[] { text });
			} else {
				// It's on the same thread, no need for Invoke
				this.chatBox.AppendText(text + Environment.NewLine);
			}
		}
		delegate void RemoveNameCallBack(string text);
		// This method is passed in to the SetTextCallBack delegate
		// to set the Text property of textBox1.
		private void DelName(string text) {
			this.nameBox.Items.Remove(text);
		}

		private void connectToolStripMenuItem_Click(object sender, EventArgs e) {
			connectToolStripMenuItem.Enabled = false;
			//CalcID = new byte[] { 0x00, 0x0B, 0x00, 0xB1, 0xE5 };
			//c = new CALCnet("IRCHub", "testname", "gcnhub.cemetech.net", 4295, CalcID);
			Connect con = new Connect();
			con.ShowDialog();
			if (con.calcnet == null) {
				connectToolStripMenuItem.Enabled = true;
				return;
			}
			c = con.calcnet;
			UserName = con.username;
			UserNameBytes = new List<byte>();
			UserNameBytes.Add(171);
			UserNameBytes.AddRange(UserName.ToByteArray(8));
			try {
				broadcast = new BroadcastPacket(c.CalcID, UserNameBytes.ToArray());
				c.SendPacket(broadcast);
				BroadcastTimer.Start();

				recvThread = new Thread(new ThreadStart(GetData));
				recvThread.Start();
			} catch (Exception ex) {
				if (recvThread != null) recvThread.Abort();
				connectToolStripMenuItem.Enabled = true;
				Write(ex.Message);
			}
		}

		private void GetData() {
			try {
				while (true) {
					CALCnetPacket packet = null;
					try {
						packet = c.GetData();
					} catch (Exception e) {
						Write(string.Concat("Could not get packet: ", e.Message));
					}
					if (packet != null) {
						StringBuilder sb = new StringBuilder();
						switch (packet.Payload[14]) {
							case 171:
								string sid = ((byte[])(packet.Payload.ToArray().SubArray(7, 5))).AsString();
								string name = ASCIIEncoding.ASCII.GetString(packet.Payload.ToArray(), 15, packet.Payload.Count - 16).TrimEnd('\0');
								if (!users.ContainsKey(sid)) {
									lock (userLock) {
										users.Add(sid, new User(name, DateTime.Now));
									}
									//nameBox.Items.Add(name);
									AddName(name);
									sb.Append(string.Format("[{0}] ** {1} entered.", DateTime.Now.ToString("hh:mm:ss"), name));
								} else {
									string oldName = users[sid].name;
									if (!oldName.Equals(name)) {
										lock (userLock) {
											users[sid].name = name;
										}
										RemoveName(oldName);
										AddName(name);
										sb.Append(string.Format("[{0}] ** {1} changed name to {2}.", DateTime.Now.ToString("hh:mm:ss"), oldName, name));
									}
									lock (userLock) {
										users[sid].lastCheckInTime = DateTime.Now;
									}
								}
								break;
							case 172:
								sid = ASCIIEncoding.ASCII.GetString(packet.Payload.ToArray(), 7, 5);
								if (users.ContainsKey(sid)) {
									name = users[sid].name;
									sb.Append(string.Format("[{0}] ** {1} parted.", DateTime.Now.ToString("hh:mm:ss"), name));
								}
								break;
							case 173:
								sb.Append(string.Format("[{0}] ", DateTime.Now.ToString("hh:mm:ss")));
								sb.Append(ASCIIEncoding.ASCII.GetString(packet.Payload.ToArray(), 15, packet.Payload.Count - 16));
								break;
						}
						if (sb.Length != 0)
							Write(sb.ToString());

						List<string> keysToDelete = new List<string>();
						foreach (string key in users.Keys) {
							if (DateTime.Now.Subtract(users[key].lastCheckInTime).Duration() > new TimeSpan(0, 0, 60)) {
								keysToDelete.Add(key);
							}
						}

						for (int i = 0; i < keysToDelete.Count; i++) {
							RemoveName(users[keysToDelete[i]].name);
							Write(string.Format("[{0}] ** {1} timed out.", DateTime.Now.ToString("hh:mm:ss"), users[keysToDelete[i]].name));
							users.Remove(keysToDelete[i]);
						}

						Thread.Sleep(100);
					} else {
						Write("Null packet received.");
						Thread.Sleep(1500);
					}
				}
			} catch (Exception e) {
				Write(string.Concat("Could not get packet: ", e.Message));
			}
		}

		private void BroadcastTimer_Tick(object sender, EventArgs e) {
			try {
				c.SendPacket(broadcast);
			} catch (Exception ex) {
				Write(string.Concat("Could not send broadcast ping: ", ex.Message));
			}
		}

		private void Chat_FormClosing(object sender, FormClosingEventArgs e) {
			if (recvThread != null) recvThread.Abort();
			lock (userLock) {
				foreach (string sid in users.Keys) {
					c.SendDirect(sid.ToPackedByteArray(), new byte[] { 172 });
				}
			}
		}

		private void inputBox_KeyPress(object sender, KeyPressEventArgs e) {
			if (e.KeyChar == '\r') {
				Submit(inputBox.Text);
				inputBox.Text = "";
				e.Handled = true;
			}
		}

		private void Submit(string data) {
			if (string.IsNullOrEmpty(data))
				return;
			Write((string.Format("[{0}] {1}:{2}", DateTime.Now.ToString("hh:mm:ss"), UserName, data)));
			List<byte> byteData = new List<byte>();
			byteData.Add(173);
			byteData.AddRange(UserName.ToByteArray());
			byteData.AddRange((":" + data).ToByteArray());
			lock (userLock) {
				foreach (string sid in users.Keys) {
					//DirectPacket p = new DirectPacket(CalcID, sid.ToPackedByteArray(), byteData);
					//c.SendData(p);
					c.SendDirect(sid.ToPackedByteArray(), byteData);
				}
			}
		}

		private void chatBox_MouseMove(object sender, MouseEventArgs e) {
			if (e.Button == System.Windows.Forms.MouseButtons.Left && !string.IsNullOrEmpty(chatBox.SelectedText)) {
				Clipboard.SetText(chatBox.SelectedText);
			}
		}
	}
}
